import {
  BadRequestException,
  Body,
  Controller,
  Get,
  NotFoundException,
  Param,
  Patch,
  Post,
  UseGuards
} from "@nestjs/common";
import { CreateStockDto, EditStockDto } from "src/stocks/stocks.dto";
import { StocksService } from "src/stocks/stocks.service";
import { error } from "winston";
import { AdminGuard, UserGuard } from "src/auth/auth.guard";

@Controller('stocks')
export class StocksController {
  constructor(
    private readonly stocks: StocksService
  ) {}

  @UseGuards(AdminGuard)
  @Post('add')
  async addStock(@Body() stock: CreateStockDto) {
    try {
      await this.stocks.createStock(stock.productId, stock.quantity);
    } catch (error) {
      throw new BadRequestException(error.message);
    }
  }

  @Get('more/:limit')
  async getProductsMoreStock(@Param('limit') limit: number) {
    try {
      const products = await this.stocks.getProductsRankedByStock(limit || 10, false);
      return products
    } catch (error) {
      throw new BadRequestException(error.message);
    }
  }

  @Get('less/:limit')
  async getProductsLessStock(@Param('limit') limit: number) {
    try {
      const products = await this.stocks.getProductsRankedByStock(limit || 10, true);
      return products
    } catch (error) {
      throw new BadRequestException(error.message);
    }

  }

  @UseGuards(AdminGuard)
  @Patch(':productId')
  async editStock(@Param('productId') id: string, @Body() stock: EditStockDto) {
    try {
      await this.stocks.alterStock(id, stock.quantity);
    } catch (error) {
      throw new BadRequestException(error.message);
    }
  }

  @UseGuards(UserGuard)
  @Get(':productId')
  async getStock(@Param('productId') productId: string) {
    try {
      const stock = await this.stocks.getCurrentStock(productId);
      return stock;
    } catch (error) {
      throw new NotFoundException(error.message);
    }
  }
}
